@@ -109,6 +109,7 @@ XrdSecService *XrdHttpProtocol::CIA = 0; // Authentication Server
 int XrdHttpProtocol::m_bio_type = 0; // BIO type identifier for our custom BIO.
 BIO_METHOD *XrdHttpProtocol::m_bio_method = NULL; // BIO method constructor.
 char *XrdHttpProtocol::xrd_cslist = nullptr;
+XrdHttpChecksumHandler XrdHttpProtocol::cksumHandler = XrdHttpChecksumHandler();
 
 XrdSysTrace XrdHttpTrace("http");
 
@@ -279,6 +280,7 @@ XrdProtocol *XrdHttpProtocol::Match(XrdLink *lp) {
   // that is is https without invoking TLS on the actual link. Eventually,
   // we should just use the link's TLS native implementation.
   //
+  SecEntity.addrInfo = lp->AddrInfo();
   XrdNetAddr *netP = const_cast<XrdNetAddr*>(lp->NetAddr());
   netP->SetDialect("https");
   netP->SetTLS(true);
@@ -526,22 +528,23 @@ int XrdHttpProtocol::Process(XrdLink *lp) // We ignore the argument here
       TRACEI(DEBUG, " Entering SSL_accept...");
       int res = SSL_accept(ssl);
       TRACEI(DEBUG, " SSL_accept returned :" << res);
-      ERR_print_errors(sslbio_err);
-
       if ((res == -1) && (SSL_get_error(ssl, res) == SSL_ERROR_WANT_READ)) {
           TRACEI(DEBUG, " SSL_accept wants to read more bytes... err:" << SSL_get_error(ssl, res));
           return 1;
         }
 
-      if (res < 0) {
+      if(res <= 0) {
           ERR_print_errors(sslbio_err);
-          SSL_free(ssl);
-          ssl = 0;
-          return -1;
-        }
+          if (res < 0) {
+
+              SSL_free(ssl);
+              ssl = 0;
+              return -1;
+          }
+      }
 
       BIO_set_nbio(sbio, 0);
-      ERR_print_errors(sslbio_err);
+
       strcpy(SecEntity.prot, "https");
 
       // Get the voms string and auth information
@@ -948,6 +951,19 @@ int XrdHttpProtocol::Config(const char *ConfigFN, XrdOucEnv *myEnv) {
   char *var;
   int cfgFD, GoNo, NoGo = 0, ismine;
 
+  cksumHandler.configure(xrd_cslist);
+  auto nonIanaChecksums = cksumHandler.getNonIANAConfiguredCksums();
+  if(nonIanaChecksums.size()) {
+      std::stringstream warningMsgSS;
+      warningMsgSS << "Config warning: the following checksum algorithms are not IANA compliant: [";
+      std::string unknownCksumString;
+      for(auto unknownCksum: nonIanaChecksums) {
+          unknownCksumString += unknownCksum + ",";
+      }
+      unknownCksumString.erase(unknownCksumString.size() - 1);
+      warningMsgSS << unknownCksumString << "]" << ". They therefore cannot be queried by a user via HTTP." ;
+      eDest.Say(warningMsgSS.str().c_str());
+  }
 
   // Initialize our custom BIO type.
   if (!m_bio_type) {
@@ -1271,7 +1287,7 @@ int XrdHttpProtocol::getDataOneShot(int blen, bool wait) {
 
 
   // Check for buffer overflow first
-  maxread = min(blen, BuffAvailable());
+  maxread = std::min(blen, BuffAvailable());
   TRACE(DEBUG, "getDataOneShot BuffAvailable: " << BuffAvailable() << " maxread: " << maxread);
 
   if (!maxread)
@@ -1283,7 +1299,7 @@ int XrdHttpProtocol::getDataOneShot(int blen, bool wait) {
     if (!wait) {
       int l = SSL_pending(ssl);
       if (l > 0)
-        sslavail = min(maxread, SSL_pending(ssl));
+        sslavail = std::min(maxread, SSL_pending(ssl));
     }
 
     if (sslavail < 0) {
@@ -1456,10 +1472,10 @@ int XrdHttpProtocol::BuffgetData(int blen, char **data, bool wait) {
   // And now make available the data taken from the buffer. Note that the buffer
   // may be empty...
   if (myBuffStart <= myBuffEnd) {
-    rlen = min( (long) blen, (long)(myBuffEnd - myBuffStart) );
+    rlen = std::min( (long) blen, (long)(myBuffEnd - myBuffStart) );
 
   } else
-    rlen = min( (long) blen, (long)(myBuff->buff + myBuff->bsize - myBuffStart) );
+    rlen = std::min( (long) blen, (long)(myBuff->buff + myBuff->bsize - myBuffStart) );
 
   *data = myBuffStart;
   BuffConsume(rlen);
@@ -1515,6 +1531,7 @@ int XrdHttpProtocol::StartSimpleResp(int code, const char *desc, const char *hea
     else if (code == 404) ss << "Not Found";
     else if (code == 405) ss << "Method Not Allowed";
     else if (code == 500) ss << "Internal Server Error";
+    else if (code == 400) ss << "Bad Request";
     else ss << "Unknown";
   }
   ss << crlf;
@@ -1528,7 +1545,7 @@ int XrdHttpProtocol::StartSimpleResp(int code, const char *desc, const char *hea
   if ((bodylen >= 0) && (code != 100))
     ss << "Content-Length: " << bodylen << crlf;
 
-  if (header_to_add)
+  if (header_to_add && (header_to_add[0] != '\0'))
     ss << header_to_add << crlf;
 
   ss << crlf;
@@ -1545,17 +1562,17 @@ int XrdHttpProtocol::StartSimpleResp(int code, const char *desc, const char *hea
 /*                      S t a r t C h u n k e d R e s p                       */
 /******************************************************************************/
   
-int XrdHttpProtocol::StartChunkedResp(int code, const char *desc, const char *header_to_add, bool keepalive) {
+int XrdHttpProtocol::StartChunkedResp(int code, const char *desc, const char *header_to_add, long long bodylen, bool keepalive) {
   const std::string crlf = "\r\n";
-
   std::stringstream ss;
-  if (header_to_add) {
+
+  if (header_to_add && (header_to_add[0] != '\0')) {
     ss << header_to_add << crlf;
   }
-  ss << "Transfer-Encoding: chunked";
 
+  ss << "Transfer-Encoding: chunked";
   TRACEI(RSP, "Starting chunked response");
-  return StartSimpleResp(code, desc, ss.str().c_str(), -1, keepalive);
+  return StartSimpleResp(code, desc, ss.str().c_str(), bodylen, keepalive);
 }
 
 /******************************************************************************/
@@ -1563,23 +1580,37 @@ int XrdHttpProtocol::StartChunkedResp(int code, const char *desc, const char *he
 /******************************************************************************/
   
 int XrdHttpProtocol::ChunkResp(const char *body, long long bodylen) {
+  long long content_length = (bodylen <= 0) ? (body ? strlen(body) : 0) : bodylen;
+  if (ChunkRespHeader(content_length))
+    return -1;
+
+  if (body && SendData(body, content_length))
+    return -1;
+
+  return ChunkRespFooter();
+}
+
+/******************************************************************************/
+/*                       C h u n k R e s p H e a d e r                        */
+/******************************************************************************/
+
+int XrdHttpProtocol::ChunkRespHeader(long long bodylen) {
   const std::string crlf = "\r\n";
-  long long chunk_length = bodylen;
-  if (bodylen <= 0) {
-    chunk_length = body ? strlen(body) : 0;
-  }
   std::stringstream ss;
 
-  ss << std::hex << chunk_length << std::dec << crlf;
+  ss << std::hex << bodylen << std::dec << crlf;
 
   const std::string &chunkhdr = ss.str();
-  TRACEI(RSP, "Sending encoded chunk of size " << chunk_length);
-  if (SendData(chunkhdr.c_str(), chunkhdr.size()))
-    return -1;
+  TRACEI(RSP, "Sending encoded chunk of size " << bodylen);
+  return (SendData(chunkhdr.c_str(), chunkhdr.size())) ? -1 : 0;
+}
 
-  if (body && SendData(body, chunk_length))
-    return -1;
+/******************************************************************************/
+/*                       C h u n k R e s p F o o t e r                        */
+/******************************************************************************/
 
+int XrdHttpProtocol::ChunkRespFooter() {
+  const std::string crlf = "\r\n";
   return (SendData(crlf.c_str(), crlf.size())) ? -1 : 0;
 }
 
@@ -1745,11 +1776,26 @@ void XrdHttpProtocol::Cleanup() {
   }
 
   if (ssl) {
-
-
-    if (SSL_shutdown(ssl) != 1) {
-      TRACE(ALL, " SSL_shutdown failed");
-      ERR_print_errors(sslbio_err);
+    // Shutdown the SSL/TLS connection
+    // https://www.openssl.org/docs/man1.0.2/man3/SSL_shutdown.html
+    // We don't need a bidirectional shutdown as
+    // when we are here, the connection will not be re-used.
+    // In the case SSL_shutdown returns 0,
+    // "the output of SSL_get_error(3) may be misleading, as an erroneous SSL_ERROR_SYSCALL may be flagged even though no error occurred."
+    // we will then just flush the thread's queue.
+    // In the case an error really happened, we print the error that happened
+    int ret = SSL_shutdown(ssl);
+    if (ret != 1) {
+        if(ret == 0) {
+            // Clean this thread's error queue for the old openssl versions
+            #if OPENSSL_VERSION_NUMBER < 0x10100000L
+                ERR_remove_thread_state(nullptr);
+            #endif
+        } else {
+            //ret < 0, an error really happened.
+            TRACE(ALL, " SSL_shutdown failed");
+            ERR_print_errors(sslbio_err);
+        }
     }
 
     if (secxtractor)
