@@ -55,6 +55,7 @@
 #include <locale>
 #include <string>
 #include "XrdOuc/XrdOucTUtils.hh"
+#include "XrdOuc/XrdOucUtils.hh"
 
 #include "XrdHttpUtils.hh"
 
@@ -71,122 +72,9 @@ namespace
 const char *TraceID = "Req";
 }
 
-static std::string convert_digest_rfc_name(const std::string &rfc_name_multiple)
-{
-  std::stringstream rfc_name_multiple_ss;
-  rfc_name_multiple_ss << rfc_name_multiple;
-  for (std::string rfc_name; std::getline(rfc_name_multiple_ss, rfc_name, ','); ) {
-    rfc_name.erase(rfc_name.find_last_not_of(" \n\r\t") + 1);
-    rfc_name.erase(0, rfc_name.find_first_not_of(" \n\r\t"));
-    rfc_name = rfc_name.substr(0, rfc_name.find(";"));
-    if (!strcasecmp(rfc_name.c_str(), "md5")) {
-      return "md5";
-    } else if (!strcasecmp(rfc_name.c_str(), "adler32")) {
-      return "adler32";
-    } else if (!strcasecmp(rfc_name.c_str(), "SHA")) {
-      return "SHA";
-    } else if (!strcasecmp(rfc_name.c_str(), "SHA-256")) {
-      return "SHA-256";
-    } else if (!strcasecmp(rfc_name.c_str(), "SHA-512")) {
-      return "SHA-512";
-    } else if (!strcasecmp(rfc_name.c_str(), "UNIXcksum")) {
-      return "UNIXcksum";
-    } else if (!strcasecmp(rfc_name.c_str(), "crc32")) {
-        return "crc32";
-    } else if (!strcasecmp(rfc_name.c_str(), "crc32c")) {
-        return "crc32c";
-    }
-  }
-  return "unknown";
-}
-
-
-static XrdOucString convert_digest_name(const std::string &rfc_name_multiple)
-{
-  std::stringstream rfc_name_multiple_ss;
-  rfc_name_multiple_ss << rfc_name_multiple;
-  for (std::string rfc_name; std::getline(rfc_name_multiple_ss, rfc_name, ','); ) {
-    rfc_name.erase(rfc_name.find_last_not_of(" \n\r\t") + 1);
-    rfc_name.erase(0, rfc_name.find_first_not_of(" \n\r\t"));
-    rfc_name = rfc_name.substr(0, rfc_name.find(";"));
-    if (!strcasecmp(rfc_name.c_str(), "md5")) {
-      return "md5";
-    } else if (!strcasecmp(rfc_name.c_str(), "adler32")) {
-      return "adler32";
-    } else if (!strcasecmp(rfc_name.c_str(), "SHA")) {
-      return "sha1";
-    } else if (!strcasecmp(rfc_name.c_str(), "SHA-256")) {
-      return "sha256";
-    } else if (!strcasecmp(rfc_name.c_str(), "SHA-512")) {
-      return "sha512";
-    } else if (!strcasecmp(rfc_name.c_str(), "UNIXcksum")) {
-      return "cksum";
-    } else if (!strcasecmp(rfc_name.c_str(), "crc32")) {
-      return "crc32";
-    } else if (!strcasecmp(rfc_name.c_str(), "crc32c")) {
-      return "crc32c";
-    }
-  }
-  return "unknown";
-}
-
-
-static std::string convert_xrootd_to_rfc_name(const std::string &xrootd_name)
-{
-  if (!strcasecmp(xrootd_name.c_str(), "md5")) {
-    return "md5";
-  } else if (!strcasecmp(xrootd_name.c_str(), "adler32")) {
-    return "adler32";
-  } else if (!strcasecmp(xrootd_name.c_str(), "sha1")) {
-    return "SHA";
-  } else if (!strcasecmp(xrootd_name.c_str(), "sha256")) {
-    return "SHA-256";
-  } else if (!strcasecmp(xrootd_name.c_str(), "sha512")) {
-    return "SHA-512";
-  } else if (!strcasecmp(xrootd_name.c_str(), "cksum")) {
-    return "UNIXcksum";
-  } else if (!strcasecmp(xrootd_name.c_str(), "crc32")) {
-    return "crc32";
-  } else if (!strcasecmp(xrootd_name.c_str(), "crc32c")) {
-    return "crc32c";
-  }
-  return "unknown";
-}
-
-
-static bool needs_base64_padding(const std::string &rfc_name)
-{
-  if (!strcasecmp(rfc_name.c_str(), "md5")) {
-    return true;
-  } else if (!strcasecmp(rfc_name.c_str(), "adler32")) {
-    return false;
-  } else if (!strcasecmp(rfc_name.c_str(), "SHA")) {
-    return true;
-  } else if (!strcasecmp(rfc_name.c_str(), "SHA-256")) {
-    return true;
-  } else if (!strcasecmp(rfc_name.c_str(), "SHA-512")) {
-    return true;
-  } else if (!strcasecmp(rfc_name.c_str(), "UNIXcksum")) {
-    return false;
-  } else if (!strcasecmp(rfc_name.c_str(), "crc32")) {
-    return false;
-  } else if (!strcasecmp(rfc_name.c_str(), "crc32c")) {
-    return true;
-  }
-  return false;
-}
-
-
 void trim(std::string &str)
 {
-  // Trim leading non-letters
-  while( str.size() && !isgraph(str[0]) ) str.erase(str.begin());
-
-  // Trim trailing non-letters
-  
-  while( str.size() && !isgraph(str[str.size()-1]) )
-    str.resize (str.size () - 1);
-
+    XrdOucUtils::trim(str);
 }
 
 
@@ -198,7 +86,7 @@ std::string ISOdatetime(time_t t) {
   gmtime_r(&t, &t1);
 
   strftime(datebuf, 127, "%a, %d %b %Y %H:%M:%S GMT", &t1);
-  return (string) datebuf;
+  return (std::string) datebuf;
 
 }
 
@@ -296,13 +184,16 @@ int XrdHttpReq::parseLine(char *line, int len) {
 
     } else if (!strcmp(key, "Expect") && strstr(val, "100-continue")) {
       sendcontinue = true;
-    } else if (!strcasecmp(key, "Transfer-Encoding") && strstr(val, "chunked")) {
+    } else if (!strcasecmp(key, "TE") && strstr(val, "trailers")) {
+      m_trailer_headers = true;
+    } else if (!strcasecmp(key, "X-Transfer-Status") && strstr(val, "true")) {
       m_transfer_encoding_chunked = true;
+      m_status_trailer = true;
     } else {
       // Some headers need to be translated into "local" cgi info.
-      std::map< std:: string, std:: string > ::iterator it = prot->hdr2cgimap.find(key);
+      std::map< std::string, std::string > ::iterator it = prot->hdr2cgimap.find(key);
       if (it != prot->hdr2cgimap.end() && (opaque ? (0 == opaque->Get(it->second.c_str())) : true)) {
-        std:: string s;
+        std::string s;
         s.assign(val, line+len-val);
         trim(s);
 
@@ -389,13 +280,13 @@ int XrdHttpReq::parseRWOp(char *str) {
     kXR_int32 newlen = sz;
 
     if (filesize > 0)
-      newlen = (kXR_int32) min(filesize - o1.bytestart, sz);
+      newlen = (kXR_int32) std::min(filesize - o1.bytestart, sz);
 
     rwOps.push_back(o1);
 
     while (len_ok < newlen) {
       ReadWriteOp nfo;
-      int len = min(newlen - len_ok, READV_MAXCHUNKSIZE);
+      int len = std::min(newlen - len_ok, READV_MAXCHUNKSIZE);
 
       nfo.bytestart = o1.bytestart + len_ok;
       nfo.byteend = nfo.bytestart + len - 1;
@@ -587,7 +478,7 @@ int XrdHttpReq::ReqReadV() {
 }
 
 std::string XrdHttpReq::buildPartialHdr(long long bytestart, long long byteend, long long fsz, char *token) {
-  ostringstream s;
+  std::ostringstream s;
 
   s << "\r\n--" << token << "\r\n";
   s << "Content-type: text/plain; charset=UTF-8\r\n";
@@ -597,7 +488,7 @@ std::string XrdHttpReq::buildPartialHdr(long long bytestart, long long byteend,
 }
 
 std::string XrdHttpReq::buildPartialHdrEnd(char *token) {
-  ostringstream s;
+  std::ostringstream s;
 
   s << "\r\n--" << token << "--\r\n";
 
@@ -1016,6 +907,9 @@ void XrdHttpReq::mapXrdErrorToHttpStatus() {
   // Do error mapping
   if (xrdresp == kXR_error) {
     switch (xrderrcode) {
+      case kXR_AuthFailed:
+        httpStatusCode = 401; httpStatusText = "Unauthorized";
+        break;
       case kXR_NotAuthorized:
         httpStatusCode = 403; httpStatusText = "Operation not permitted";
         break;
@@ -1032,7 +926,14 @@ void XrdHttpReq::mapXrdErrorToHttpStatus() {
         httpStatusCode = 409; httpStatusText = "Resource is a directory";
         break;
       case kXR_ItExists:
-        httpStatusCode = 409; httpStatusText = "File already exists";
+        if(request != ReqType::rtDELETE) {
+          httpStatusCode = 409; httpStatusText = "File already exists";
+        } else {
+          // In the case the XRootD layer returns a kXR_ItExists after a deletion
+          // was submitted, we return a 405 status code with the error message set by
+          // the XRootD layer
+          httpStatusCode = 405;
+        }
         break;
       case kXR_InvalidRequest:
         httpStatusCode = 405; httpStatusText = "Method is not allowed";
@@ -1047,45 +948,12 @@ void XrdHttpReq::mapXrdErrorToHttpStatus() {
                  << "] to status code [" << httpStatusCode << "]");
 
     httpStatusText += "\n";
+  } else {
+      httpStatusCode = 200;
+      httpStatusText = "OK";
   }
 }
 
-/**
- * Select the checksum to be computed depending on the userDigest passed in parameter
- * @param userDigest the digest request from the user (extracted from the Want-Digest header)
- * @param selectedChecksum the checksum that will be performed
- */
-void XrdHttpReq::selectChecksum(const std::string &userDigest, std::string & selectedChecksum) {
-    char * configChecksumList = XrdHttpProtocol::xrd_cslist;
-    selectedChecksum = "unknown";
-    if(configChecksumList != nullptr) {
-        //The env variable is set, some checksums have been configured
-        std::vector<std::string> userDigestsVec;
-        XrdOucTUtils::splitString(userDigestsVec,userDigest,",");
-        std::vector<std::string> configChecksums;
-        XrdOucTUtils::splitString(configChecksums,configChecksumList,",");
-        selectedChecksum = configChecksums[0];
-        auto configChecksumItor = configChecksums.end();
-        std::find_if(userDigestsVec.begin(), userDigestsVec.end(), [&configChecksums, &configChecksumItor](const std::string & userDigest){
-            configChecksumItor = std::find_if(configChecksums.begin(),configChecksums.end(),[&userDigest](const std::string & configChecksum){
-                std::string userDigestTrimmed = userDigest;
-                trim(userDigestTrimmed);
-                if(configChecksum.find(userDigestTrimmed) != std::string::npos) {
-                    return true;
-                }
-                return false;
-            });
-            return configChecksumItor != configChecksums.end();
-        });
-        //By default, the selected checksum is the first one of the configured checksum list.
-        // If the user gave a checksum that do not exist, then the checksum returned will be the default one
-        configChecksumItor = configChecksumItor != configChecksums.end() ? configChecksumItor : configChecksums.begin();
-        std::vector<std::string> checksumIdName;
-        XrdOucTUtils::splitString(checksumIdName,*configChecksumItor,":");
-        selectedChecksum = checksumIdName[1];
-    }
-}
-
 int XrdHttpReq::ProcessHTTPReq() {
 
   kXR_int32 l;
@@ -1159,15 +1027,19 @@ int XrdHttpReq::ProcessHTTPReq() {
         const char *opaque = strchr(resourceplusopaque.c_str(), '?');
         // Note that doChksum requires that the memory stays alive until the callback is invoked.
         m_resource_with_digest = resourceplusopaque;
-        std::string selectedChecksum;
-        selectChecksum(m_req_digest,selectedChecksum);
-        m_req_digest = convert_digest_name(selectedChecksum).c_str();
+
+        m_req_cksum = prot->cksumHandler.getChecksumToRun(m_req_digest);
+        if(!m_req_cksum) {
+            // No HTTP IANA checksums have been configured by the server admin, return a "METHOD_NOT_ALLOWED" error
+            prot->SendSimpleResp(403, NULL, NULL, (char *) "No HTTP-IANA compatible checksums have been configured.", 0, false);
+            return -1;
+        }
         if (!opaque) {
           m_resource_with_digest += "?cks.type=";
-          m_resource_with_digest += m_req_digest.c_str();
+          m_resource_with_digest += m_req_cksum->getXRootDConfigDigestName().c_str();
         } else {
           m_resource_with_digest += "&cks.type=";
-          m_resource_with_digest += m_req_digest.c_str();
+          m_resource_with_digest += m_req_cksum->getXRootDConfigDigestName().c_str();
         }
         if (prot->doChksum(m_resource_with_digest) < 0) {
           // In this case, the Want-Digest header was set and PostProcess gave the go-ahead to do a checksum.
@@ -1283,7 +1155,7 @@ int XrdHttpReq::ProcessHTTPReq() {
             }
 
 
-            string res;
+            std::string res;
             res = resourceplusopaque.c_str();
             //res += "?xrd.dirstat=1";
 
@@ -1306,13 +1178,19 @@ int XrdHttpReq::ProcessHTTPReq() {
             // In this case, the Want-Digest header was set.
             bool has_opaque = strchr(resourceplusopaque.c_str(), '?');
             // Note that doChksum requires that the memory stays alive until the callback is invoked.
+            m_req_cksum = prot->cksumHandler.getChecksumToRun(m_req_digest);
+            if(!m_req_cksum) {
+                // No HTTP IANA checksums have been configured by the server admin, return a "METHOD_NOT_ALLOWED" error
+                prot->SendSimpleResp(403, NULL, NULL, (char *) "No HTTP-IANA compatible checksums have been configured.", 0, false);
+                return -1;
+            }
             m_resource_with_digest = resourceplusopaque;
             if (has_opaque) {
               m_resource_with_digest += "&cks.type=";
-              m_resource_with_digest += convert_digest_name(m_req_digest);
+              m_resource_with_digest += m_req_cksum->getXRootDConfigDigestName().c_str();
             } else {
               m_resource_with_digest += "?cks.type=";
-              m_resource_with_digest += convert_digest_name(m_req_digest);
+              m_resource_with_digest += m_req_cksum->getXRootDConfigDigestName().c_str();
             }
             if (prot->doChksum(m_resource_with_digest) < 0) {
               prot->SendSimpleResp(500, NULL, NULL, (char *) "Failed to start internal checksum request to satisfy Want-Digest header.", 0, false);
@@ -1406,18 +1284,20 @@ int XrdHttpReq::ProcessHTTPReq() {
             xrdreq.read.dlen = 0;
             
             if (rwOps.size() == 0) {
-              l = (long)min(filesize-writtenbytes, (long long)1024*1024);
+              l = (long)std::min(filesize-writtenbytes, (long long)1024*1024);
               offs = writtenbytes;
               xrdreq.read.offset = htonll(writtenbytes);
               xrdreq.read.rlen = htonl(l);
             } else {
-              l = min(rwOps[0].byteend - rwOps[0].bytestart + 1 - writtenbytes, (long long)1024*1024);
+              l = std::min(rwOps[0].byteend - rwOps[0].bytestart + 1 - writtenbytes, (long long)1024*1024);
               offs = rwOps[0].bytestart + writtenbytes;
               xrdreq.read.offset = htonll(offs);
               xrdreq.read.rlen = htonl(l);
             }
 
-            if (prot->ishttps) {
+            // If we are using HTTPS or if the client requested trailers, disable sendfile
+            // (in the latter case, the chunked encoding prevents sendfile usage)
+            if (prot->ishttps || (m_transfer_encoding_chunked && m_trailer_headers)) {
               if (!prot->Bridge->setSF((kXR_char *) fhandle, false)) {
                 TRACE(REQ, " XrdBridge::SetSF(false) failed.");
 
@@ -1573,7 +1453,7 @@ int XrdHttpReq::ProcessHTTPReq() {
             memcpy(xrdreq.write.fhandle, fhandle, 4);
 
             long long chunk_bytes_remaining = m_current_chunk_size - m_current_chunk_offset;
-            long long bytes_to_write = min(static_cast<long long>(prot->BuffUsed()),
+            long long bytes_to_write = std::min(static_cast<long long>(prot->BuffUsed()),
                                            chunk_bytes_remaining);
 
             xrdreq.write.offset = htonll(writtenbytes);
@@ -1596,7 +1476,7 @@ int XrdHttpReq::ProcessHTTPReq() {
           xrdreq.write.requestid = htons(kXR_write);
           memcpy(xrdreq.write.fhandle, fhandle, 4);
 
-          long long bytes_to_read = min(static_cast<long long>(prot->BuffUsed()),
+          long long bytes_to_read = std::min(static_cast<long long>(prot->BuffUsed()),
                                         length - writtenbytes);
 
           xrdreq.write.offset = htonll(writtenbytes);
@@ -1660,7 +1540,7 @@ int XrdHttpReq::ProcessHTTPReq() {
           // --------- STAT is always the first step
           memset(&xrdreq, 0, sizeof (ClientRequest));
           xrdreq.stat.requestid = htons(kXR_stat);
-          string s = resourceplusopaque.c_str();
+          std::string s = resourceplusopaque.c_str();
 
 
           l = resourceplusopaque.length() + 1;
@@ -1681,7 +1561,7 @@ int XrdHttpReq::ProcessHTTPReq() {
             memset(&xrdreq, 0, sizeof (ClientRequest));
             xrdreq.rmdir.requestid = htons(kXR_rmdir);
 
-            string s = resourceplusopaque.c_str();
+            std::string s = resourceplusopaque.c_str();
 
             l = s.length() + 1;
             xrdreq.rmdir.dlen = htonl(l);
@@ -1695,7 +1575,7 @@ int XrdHttpReq::ProcessHTTPReq() {
             memset(&xrdreq, 0, sizeof (ClientRequest));
             xrdreq.rm.requestid = htons(kXR_rm);
 
-            string s = resourceplusopaque.c_str();
+            std::string s = resourceplusopaque.c_str();
 
             l = s.length() + 1;
             xrdreq.rm.dlen = htonl(l);
@@ -1754,7 +1634,7 @@ int XrdHttpReq::ProcessHTTPReq() {
           // --------- STAT is always the first step
           memset(&xrdreq, 0, sizeof (ClientRequest));
           xrdreq.stat.requestid = htons(kXR_stat);
-          string s = resourceplusopaque.c_str();
+          std::string s = resourceplusopaque.c_str();
 
 
           l = resourceplusopaque.length() + 1;
@@ -1785,7 +1665,7 @@ int XrdHttpReq::ProcessHTTPReq() {
           memset(&xrdreq, 0, sizeof (ClientRequest));
           xrdreq.dirlist.requestid = htons(kXR_dirlist);
 
-          string s = resourceplusopaque.c_str();
+          std::string s = resourceplusopaque.c_str();
           xrdreq.dirlist.options[0] = kXR_dstat;
           //s += "?xrd.dirstat=1";
 
@@ -1812,7 +1692,7 @@ int XrdHttpReq::ProcessHTTPReq() {
       memset(&xrdreq, 0, sizeof (ClientRequest));
       xrdreq.mkdir.requestid = htons(kXR_mkdir);
 
-      string s = resourceplusopaque.c_str();
+      std::string s = resourceplusopaque.c_str();
       xrdreq.mkdir.options[0] = (kXR_char) kXR_mkdirpath;
 
       l = s.length() + 1;
@@ -1833,7 +1713,7 @@ int XrdHttpReq::ProcessHTTPReq() {
       memset(&xrdreq, 0, sizeof (ClientRequest));
       xrdreq.mv.requestid = htons(kXR_mv);
 
-      string s = resourceplusopaque.c_str();
+      std::string s = resourceplusopaque.c_str();
       s += " ";
 
       char buf[256];
@@ -1900,9 +1780,8 @@ XrdHttpReq::PostProcessChecksum(std::string &digest_header) {
     TRACEI(REQ, "Checksum for HEAD " << resource.c_str() << " "
                << reinterpret_cast<char *>(iovP[0].iov_base) << "=" 
                << reinterpret_cast<char *>(iovP[iovN-1].iov_base));
-    std::string response_name = convert_xrootd_to_rfc_name(reinterpret_cast<char *>(iovP[0].iov_base));
 
-    bool convert_to_base64 = needs_base64_padding(convert_digest_rfc_name(m_req_digest));
+    bool convert_to_base64 = m_req_cksum->needsBase64Padding();
     char *digest_value = reinterpret_cast<char *>(iovP[iovN-1].iov_base);
     if (convert_to_base64) {
       size_t digest_length = strlen(digest_value);
@@ -1920,13 +1799,13 @@ XrdHttpReq::PostProcessChecksum(std::string &digest_header) {
     }
 
     digest_header = "Digest: ";
-    digest_header += convert_digest_rfc_name(m_req_digest);
+    digest_header += m_req_cksum->getHttpName();
     digest_header += "=";
     digest_header += digest_value;
     if (convert_to_base64) {free(digest_value);}
     return 0;
   } else {
-    prot->SendSimpleResp(500, NULL, NULL, "Underlying filesystem failed to calculate checksum.", 0, false);
+    prot->SendSimpleResp(httpStatusCode, NULL, NULL, httpStatusText.c_str(), httpStatusText.length(), false);
     return -1;
   }
 }
@@ -1973,7 +1852,7 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
           if (m_req_digest.size()) {
             return 0;
           } else {
-            prot->SendSimpleResp(200, NULL, NULL, NULL, filesize, keepalive);
+            prot->SendSimpleResp(200, NULL, "Accept-Ranges: bytes", NULL, filesize, keepalive);
             return keepalive ? 1 : -1;
           }
         }
@@ -1983,12 +1862,14 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
         return keepalive ? 1 : -1;
       } else { // We requested a checksum and now have its response.
         if (iovN > 0) {
-          std::string digest_response;
-          int response = PostProcessChecksum(digest_response);
+          std::string response_headers;
+          int response = PostProcessChecksum(response_headers);
           if (-1 == response) {
                 return -1;
           }
-          prot->SendSimpleResp(200, NULL, digest_response.c_str(), NULL, filesize, keepalive);
+          if (!response_headers.empty()) {response_headers += "\r\n";}
+          response_headers += "Accept-Ranges: bytes";
+          prot->SendSimpleResp(200, NULL, response_headers.c_str(), NULL, filesize, keepalive);
           return keepalive ? 1 : -1;
         } else {
           prot->SendSimpleResp(500, NULL, NULL, "Underlying filesystem failed to calculate checksum.", 0, false);
@@ -2078,7 +1959,7 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
 
             if (e.path.length() && (e.path != ".") && (e.path != "..")) {
               // The entry is filled. <td class="ft-file"><a href="file1.txt">file1.txt</a></td>
-              string p = "<tr>"
+              std::string p = "<tr>"
                       "<td class=\"mode\">";
 
               if (e.flags & kXR_isDir) p += "d";
@@ -2272,52 +2153,77 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
 
               getfhandle();
               
-              // Now parse the stat info if we still don't have it
-              if (filesize == 0) {
-                if (iovP[1].iov_len > 1) {
-                  TRACEI(REQ, "Stat for GET " << resource.c_str() 
-                           << " stat=" << (char *) iovP[1].iov_base);
-              
-                  long dummyl;
-                  sscanf((const char *) iovP[1].iov_base, "%ld %lld %ld %ld",
-                        &dummyl,
-                        &filesize,
-                        &fileflags,
-                        &filemodtime);
+              // Always try to parse response.  In the case of a caching proxy, the open
+              // will have created the file in cache
+              if (iovP[1].iov_len > 1) {
+                TRACEI(REQ, "Stat for GET " << resource.c_str()
+                         << " stat=" << (char *) iovP[1].iov_base);
 
-                  // As above: if the client specified a response size, we use that.
-                  // Otherwise, utilize the filesize
-                  if (!length) {
-                    length = filesize;
-                  }
+                long dummyl;
+                sscanf((const char *) iovP[1].iov_base, "%ld %lld %ld %ld",
+                      &dummyl,
+                      &filesize,
+                      &fileflags,
+                      &filemodtime);
+
+                // As above: if the client specified a response size, we use that.
+                // Otherwise, utilize the filesize
+                if (!length) {
+                  length = filesize;
                 }
-                else
-                  TRACEI(ALL, "GET returned no STAT information. Internal error?");
               }
-              
+              else {
+                TRACEI(ALL, "GET returned no STAT information. Internal error?");
+              }
+
+              std::string responseHeader;
+              if (!m_digest_header.empty()) {
+                responseHeader = m_digest_header;
+              }
+              long one;
+              if (filemodtime && XrdOucEnv::Import("XRDPFC", one)) {
+                  if (!responseHeader.empty()) {
+                    responseHeader += "\r\n";
+                  }
+                  long object_age = time(NULL) - filemodtime;
+                  responseHeader += std::string("Age: ") + std::to_string(object_age < 0 ? 0 : object_age);
+              }
+
               if (rwOps.size() == 0) {
                 // Full file.
-                
-                prot->SendSimpleResp(200, NULL, m_digest_header.empty() ? NULL : m_digest_header.c_str(), NULL, filesize, keepalive);
+
+                if (m_transfer_encoding_chunked && m_trailer_headers) {
+                  prot->StartChunkedResp(200, NULL, responseHeader.empty() ? NULL : responseHeader.c_str(), filesize, keepalive);
+                } else {
+                  prot->SendSimpleResp(200, NULL, responseHeader.empty() ? NULL : responseHeader.c_str(), NULL, filesize, keepalive);
+                }
                 return 0;
               } else
                 if (rwOps.size() == 1) {
                 // Only one read to perform
+                if (rwOps[0].byteend < 0) // The requested range was along the lines of "Range: 1234-", meaning we need to fill in the end
+                  rwOps[0].byteend = filesize - 1;
                 int cnt = (rwOps[0].byteend - rwOps[0].bytestart + 1);
                 char buf[64];
                 
                 XrdOucString s = "Content-Range: bytes ";
                 sprintf(buf, "%lld-%lld/%lld", rwOps[0].bytestart, rwOps[0].byteend, filesize);
                 s += buf;
-                if (!m_digest_header.empty()) {
-                  s += "\n";
-                  s += m_digest_header.c_str();
+                if (!responseHeader.empty()) {
+                  s += "\r\n";
+                  s += responseHeader.c_str();
                 }
 
                 prot->SendSimpleResp(206, NULL, (char *)s.c_str(), NULL, cnt, keepalive);
                 return 0;
               } else
                 if (rwOps.size() > 1) {
+                // First, check that the amount of range request can be handled by the vector read of the XRoot layer
+                if(rwOps.size() > XrdProto::maxRvecsz) {
+                  std::string errMsg = "Too many range requests provided. Maximum range requests supported is " + std::to_string(XrdProto::maxRvecsz);
+                  prot->SendSimpleResp(400, NULL, NULL,errMsg.c_str(), errMsg.size(), false);
+                  return -1;
+                }
                 // Multiple reads to perform, compose and send the header
                 int cnt = 0;
                 for (size_t i = 0; i < rwOps.size(); i++) {
@@ -2365,11 +2271,59 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
           default: //read or readv
           {
 
-            // Nothing to do if we are postprocessing a close
-            if (ntohs(xrdreq.header.requestid) == kXR_close) return keepalive ? 1 : -1;
-            
-            // Close() if this was the third state of a readv, otherwise read the next chunk
-            if ((reqstate == 3) && (ntohs(xrdreq.header.requestid) == kXR_readv)) return keepalive ? 1: -1;
+            // If we are postprocessing a close, potentially send out informational trailers
+            if ((ntohs(xrdreq.header.requestid) == kXR_close) ||
+              ((reqstate == 3) && (ntohs(xrdreq.header.requestid) == kXR_readv)))
+            {
+
+              if (m_transfer_encoding_chunked && m_trailer_headers) {
+                if (prot->ChunkRespHeader(0))
+                  return -1;
+
+                const std::string crlf = "\r\n";
+                std::stringstream ss;
+                ss << "X-Transfer-Status: " << httpStatusCode << ": " << httpStatusText << crlf;
+
+                const auto header = ss.str();
+                if (prot->SendData(header.c_str(), header.size()))
+                  return -1;
+
+                if (prot->ChunkRespFooter())
+                  return -1;
+              }
+
+              return keepalive ? 1 : -1;
+            }
+
+            // On error, we can only send out a message if trailers are enabled and the
+            // status response in trailer behavior is requested.
+            if (xrdresp == kXR_error) {
+              if (m_transfer_encoding_chunked && m_trailer_headers && m_status_trailer) {
+                // A trailer header is appropriate in this case; this is signified by
+                // a chunk with size zero, then the trailer, then a crlf.
+                //
+                // We only send the status trailer when explicitly requested; otherwise a
+                // "normal" HTTP client might simply see a short response and think it's a
+                // success
+                if (prot->ChunkRespHeader(0))
+                  return -1;
+
+                const std::string crlf = "\r\n";
+                std::stringstream ss;
+                ss << "X-Transfer-Status: " << httpStatusCode << ": " << httpStatusText << crlf;
+
+		const auto header = ss.str();
+                if (prot->SendData(header.c_str(), header.size()))
+                  return -1;
+
+                if (prot->ChunkRespFooter())
+                  return -1;
+
+                return -1;
+              } else {
+                return -1;
+              }
+            }
 
             // Prevent scenario where data is expected but none is actually read
             // E.g. Accessing files which return the results of a script
@@ -2380,8 +2334,6 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
               return -1;
             }
 
-            // If we are here it's too late to send a proper error message...
-            if (xrdresp == kXR_error) return -1;
 
             TRACEI(REQ, "Got data vectors to send:" << iovN);
             if (ntohs(xrdreq.header.requestid) == kXR_readv) {
@@ -2400,7 +2352,7 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
                   // Now we have a chunk coming from the server. This may be a partial chunk
 
                   if (rwOpPartialDone == 0) {
-                    string s = buildPartialHdr(rwOps[rwOpDone].bytestart,
+                    std::string s = buildPartialHdr(rwOps[rwOpDone].bytestart,
                             rwOps[rwOpDone].byteend,
                             filesize,
                             (char *) "123456");
@@ -2427,15 +2379,25 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
               }
 
               if (rwOpDone == rwOps.size()) {
-                string s = buildPartialHdrEnd((char *) "123456");
+                std::string s = buildPartialHdrEnd((char *) "123456");
                 if (prot->SendData((char *) s.c_str(), s.size())) return -1;
               }
 
-            } else
+            } else {
+              // Send chunked encoding header
+              if (m_transfer_encoding_chunked && m_trailer_headers) {
+                int sum = 0;
+                for (int i = 0; i < iovN; i++) sum += iovP[i].iov_len;
+                prot->ChunkRespHeader(sum);
+              }
               for (int i = 0; i < iovN; i++) {
                 if (prot->SendData((char *) iovP[i].iov_base, iovP[i].iov_len)) return -1;
                 writtenbytes += iovP[i].iov_len;
               }
+              if (m_transfer_encoding_chunked && m_trailer_headers) {
+                prot->ChunkRespFooter();
+              }
+            }
               
             // Let's make sure that we avoid sending the same data twice,
             // in the case where PostProcessHTTPReq is invoked again
@@ -2469,7 +2431,7 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
         fopened = true;
 
         // We try to completely fill up our buffer before flushing
-        prot->ResumeBytes = min(length - writtenbytes, (long long) prot->BuffAvailable());
+        prot->ResumeBytes = std::min(length - writtenbytes, (long long) prot->BuffAvailable());
 
         if (sendcontinue) {
           prot->SendSimpleResp(100, NULL, NULL, 0, 0, keepalive);
@@ -2496,7 +2458,7 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
           }
 
           // We try to completely fill up our buffer before flushing
-          prot->ResumeBytes = min(length - writtenbytes, (long long) prot->BuffAvailable());
+          prot->ResumeBytes = std::min(length - writtenbytes, (long long) prot->BuffAvailable());
 
           return 0;
         }
@@ -2607,7 +2569,7 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
               /* The entry is filled. */
 
 
-              string p;
+              std::string p;
               stringresp += "<D:response xmlns:lp1=\"DAV:\" xmlns:lp2=\"http://apache.org/dav/props/\" xmlns:lp3=\"LCGDM:\">\n";
               
               char *estr = escapeXML(e.path.c_str());
@@ -2661,7 +2623,7 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
 
           // If this was the last bunch of entries, send the buffer and empty it immediately
           if ((depth == 0) || !(e.flags & kXR_isDir)) {
-            string s = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<D:multistatus xmlns:D=\"DAV:\" xmlns:ns1=\"http://apache.org/dav/props/\" xmlns:ns0=\"DAV:\">\n";
+            std::string s = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<D:multistatus xmlns:D=\"DAV:\" xmlns:ns1=\"http://apache.org/dav/props/\" xmlns:ns0=\"DAV:\">\n";
             stringresp.insert(0, s);
             stringresp += "</D:multistatus>\n";
             prot->SendSimpleResp(207, (char *) "Multi-Status", (char *) "Content-Type: text/xml; charset=\"utf-8\"",
@@ -2723,7 +2685,7 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
                  */
 
 
-                string p = resource.c_str();
+                std::string p = resource.c_str();
                 if (*p.rbegin() != '/') p += "/";
                 
                 p += e.path;
@@ -2785,7 +2747,7 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
 
           // If this was the last bunch of entries, send the buffer and empty it immediately
           if (final_) {
-            string s = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<D:multistatus xmlns:D=\"DAV:\" xmlns:ns1=\"http://apache.org/dav/props/\" xmlns:ns0=\"DAV:\">\n";
+            std::string s = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<D:multistatus xmlns:D=\"DAV:\" xmlns:ns1=\"http://apache.org/dav/props/\" xmlns:ns0=\"DAV:\">\n";
             stringresp.insert(0, s);
             stringresp += "</D:multistatus>\n";
             prot->SendSimpleResp(207, (char *) "Multi-Status", (char *) "Content-Type: text/xml; charset=\"utf-8\"",
@@ -2888,6 +2850,9 @@ void XrdHttpReq::reset() {
 
   // Reset the state of the request's digest request.
   m_req_digest.clear();
+  m_digest_header.clear();
+  m_req_cksum = nullptr;
+
   m_resource_with_digest = "";
 
   headerok = false;
@@ -2901,6 +2866,9 @@ void XrdHttpReq::reset() {
   m_current_chunk_size = -1;
   m_current_chunk_offset = 0;
 
+  m_trailer_headers = false;
+  m_status_trailer = false;
+
   /// State machine to talk to the bridge
   reqstate = 0;
 
